@@ -1,5 +1,5 @@
+import datetime
 import json
-from datetime import datetime
 
 import httpx
 from thingy import NamesMixin, Thingy, classproperty
@@ -46,13 +46,37 @@ class JSONEncoder(json.JSONEncoder):
     def default(self, o):
         if isinstance(o, (datetime.date, datetime.datetime)):
             return o.isoformat()
+        if isinstance(o, BubbleThing):
+            return o.view("bubble")
         return json.JSONEncoder.default(self, o)
 
 
+async def raise_for_status(response):
+    try:
+        response.raise_for_status()
+    except httpx.HTTPStatusError:
+        await response.aread()
+        raise
+
+
 class AsyncClient(httpx.AsyncClient):
     _json_encoder = JSONEncoder
 
+    def __init__(self, *args, **kwargs):
+        kwargs.setdefault("event_hooks", {"response": [raise_for_status]})
+        super().__init__(*args, **kwargs)
+
+    @classmethod
+    def _dump_params(cls, params):
+        for key, value in params.items():
+            if not isinstance(value, (str, int)):
+                params[key] = json.dumps(value, cls=cls._json_encoder)
+
     async def request(self, *args, **kwargs):
+        params = kwargs.get("params")
+        if params:
+            self._dump_params(params)
+
         if "json" in kwargs:
             content = kwargs.pop("json")
             kwargs["content"] = json.dumps(content, cls=self._json_encoder)
@@ -83,7 +107,6 @@ class Cursor:
                 f"/api/1.1/obj/{self.cls.typename}",
                 params=self.params,
             )
-            response.raise_for_status()
         return response.json()["response"]
 
     @property
@@ -183,26 +206,17 @@ class BubbleThing(NamesMixin, Thingy):
             headers=cls._headers,
         )
 
-    @classmethod
-    def _dump_params(cls, params):
-        for key, value in params.items():
-            if not isinstance(value, (str, int)):
-                params[key] = json.dumps(value, cls=cls._json_encoder)
-
     @classmethod
     def get(cls, **params):
-        cls._dump_params(params)
         return Cursor(cls, params)
 
     @classmethod
     async def _get_by_id(cls, id, **params):
-        cls._dump_params(params)
         async with cls._get_client() as client:
             response = await client.get(
                 f"/api/1.1/obj/{cls.typename}/{id}",
                 params=params,
             )
-            response.raise_for_status()
 
         bubble_object = response.json()["response"]
         if bubble_object:
@@ -231,25 +245,21 @@ class BubbleThing(NamesMixin, Thingy):
         return await Join(cursor_or_cls, key)(self)
 
     async def put(self, **params):
-        self._dump_params(params)
         async with self._get_client() as client:
-            response = await client.put(
+            await client.put(
                 f"/api/1.1/obj/{self.__class__.typename}/{self._id}",
                 params=params,
-                json=self.view("bubble"),
+                json=self,
             )
-            response.raise_for_status()
         return self
 
     async def post(self, **params):
-        self._dump_params(params)
         async with self._get_client() as client:
             response = await client.post(
                 f"/api/1.1/obj/{self.__class__.typename}",
                 params=params,
-                json=self.view("bubble"),
+                json=self,
             )
-            response.raise_for_status()
             self._id = response.json()["id"]
         return self
 
